@@ -6,7 +6,7 @@ package Excel::Writer::XLSX::Package::App;
 #
 # Used in conjunction with Excel::Writer::XLSX
 #
-# Copyright 2000-2013, John McNamara, jmcnamara@cpan.org
+# Copyright 2000-2021, John McNamara, jmcnamara@cpan.org
 #
 # Documentation after __END__
 #
@@ -20,7 +20,7 @@ use Carp;
 use Excel::Writer::XLSX::Package::XMLwriter;
 
 our @ISA     = qw(Excel::Writer::XLSX::Package::XMLwriter);
-our $VERSION = '0.76';
+our $VERSION = '1.09';
 
 
 ###############################################################################
@@ -45,6 +45,7 @@ sub new {
     $self->{_part_names}    = [];
     $self->{_heading_pairs} = [];
     $self->{_properties}    = {};
+    $self->{_doc_security}  = 0;
 
     bless $self, $class;
 
@@ -73,6 +74,7 @@ sub _assemble_xml_file {
     $self->_write_company();
     $self->_write_links_up_to_date();
     $self->_write_shared_doc();
+    $self->_write_hyperlink_base();
     $self->_write_hyperlinks_changed();
     $self->_write_app_version();
 
@@ -195,9 +197,8 @@ sub _write_application {
 sub _write_doc_security {
 
     my $self = shift;
-    my $data = 0;
 
-    $self->xml_data_element( 'DocSecurity', $data );
+    $self->xml_data_element( 'DocSecurity', $self->{_doc_security} );
 }
 
 
@@ -366,6 +367,23 @@ sub _write_shared_doc {
 }
 
 
+###############################################################################
+#
+# _write_hyperlink_base()
+#
+# Write the <HyperlinkBase> element.
+#
+sub _write_hyperlink_base {
+
+    my $self = shift;
+    my $data = $self->{_properties}->{hyperlink_base};
+
+    return unless $data;
+
+    $self->xml_data_element( 'HyperlinkBase', $data );
+}
+
+
 ###############################################################################
 #
 # _write_hyperlinks_changed()
@@ -421,7 +439,7 @@ John McNamara jmcnamara@cpan.org
 
 =head1 COPYRIGHT
 
-(c) MM-MMXIIII, John McNamara.
+(c) MM-MMXXI, John McNamara.
 
 All Rights Reserved. This module is free software. It may be used, redistributed and/or modified under the same terms as Perl itself.
 
